<?php
namespace AIOSEO\Plugin\Addon\Redirects\Main\Server;

/**
 * Main class to work with server redirects.
 *
 * @since 1.0.0
 */
class Apache extends Server {
	/**
	 * %1$s is the old URL
	 * %2$s is the new URL
	 * %3$s is the redirect type
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected $urlFormat = 'Redirect %3$s "%1$s" "%2$s"';

	/**
	 * %1$s is the old URL
	 * %2$s is the redirect type
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected $urlNonTargetFormat = 'Redirect %2$s "%1$s"';

	/**
	 * %1$s is the old URL
	 * %2$s is the new URL
	 * %3$s is the redirect type
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected $regexFormat = 'RedirectMatch %3$s %1$s %2$s';

	/**
	 * %1$s is the old URL
	 * %2$s is the redirect type
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected $regexNonTargetFormat = 'RedirectMatch %2$s %1$s';

	protected $preContent = '';
	protected $postContent = '';

	public function __construct() {
		$this->preContent  = '# BEGIN AIOSEO REDIRECTS' . PHP_EOL . '<IfModule mod_rewrite.c>' . PHP_EOL . 'RewriteEngine On' . PHP_EOL;
		$this->postContent = '</IfModule>' . PHP_EOL . '# END AIOSEO REDIRECTS' . PHP_EOL;
		parent::__construct();
	}

	/**
	 * Formats a redirect for use in the export.
	 *
	 * @since 1.0.0
	 *
	 * @param  object $redirect The redirect to format.
	 * @return string           The formatted redirect.
	 */
	public function format( $redirect ) {
		// 4xx redirects don't have a target.
		$redirectType = intval( $redirect->type );
		if ( $redirectType >= 400 && $redirectType < 500 ) {
			return $this->formatNonTarget( $redirect );
		}

		return sprintf(
			$this->getFormat( $redirect->regex ),
			$redirect->source_url,
			$redirect->target_url,
			$redirect->type
		);
	}

	/**
	 * Build the redirect output for non-target status codes (4xx).
	 *
	 * @since 1.0.0
	 *
	 * @param object  $redirect The redirect data.
	 * @return string           The formatted string.
	 */
	public function formatNonTarget( $redirect ) {
		$target = $redirect->target_url;
		return sprintf(
			$this->getNonTargetFormat( $redirect->regex ),
			$target,
			$redirect->type
		);
	}

	/**
	 * Get the format the redirect needs to output.
	 *
	 * @since 1.0.0
	 *
	 * @param  boolean $regex Whether or not to use regex.
	 * @return string         The format of the redirect.
	 */
	public function getNonTargetFormat( $regex = false ) {
		return $regex ? $this->regexNonTargetFormat : $this->urlNonTargetFormat;
	}

	/**
	 * Get the test redirect.
	 *
	 * @since 1.0.0
	 *
	 * @return string The test redirect.
	 */
	protected function getAioseoRedirects() {
		return 'Redirect 301 "/' . $this->getTestRedirect() . '" "/' . $this->getTestInterceptedRedirect() . '"' . PHP_EOL;
	}
}