<?php
namespace AIOSEO\Plugin\Addon\Redirects\Utils;

/**
 * Parses the request.
 *
 * @since 1.0.0
 */
class Request {
	/**
	 * Retrieve the request URL.
	 *
	 * @since 1.0.0
	 *
	 * @return string The request URL.
	 */
	public static function getRequestUrl() {
		$url = '';

		if ( isset( $_SERVER['REQUEST_URI'] ) ) {
			$url = $_SERVER['REQUEST_URI'];
		}

		return apply_filters( 'aioseo_redirects_request_url', rawurldecode( stripslashes( $url ) ) );
	}

	/**
	 * Get the plain 'matched' URL:
	 *
	 * - Lowercase
	 * - No trailing slashes
	 *
	 * @since 1.0.0
	 *
	 * @return string The URL.
	 */
	public static function getMatchedUrl( $url ) {
		$url = explode( '?', $url );
		$url = untrailingslashit( $url[0] );

		// Return / or // as-is.
		if ( '/' !== $url ) {
			// Anything else remove the last /.
			$url = preg_replace( '@/$@', '', $url );
		}

		// URL encode.
		$decode = [
			'/',
			':',
			'[',
			']',
			'@',
			'~',
			',',
			'(',
			')',
			';',
		];

		// URL encode everything - this converts any i10n to the proper encoding.
		$url = rawurlencode( $url );

		// We also converted things we dont want encoding, such as a /. Change these back.
		foreach ( $decode as $char ) {
			$url = str_replace( rawurlencode( $char ), $char, $url );
		}

		// Lowercase everything.
		$url = aioseo()->helpers->toLowercase( $url );

		return $url ? $url : '/';
	}

	/**
	 * Get the target URL formatted.
	 *
	 * @since 1.0.0
	 *
	 * @param  string $url The URL.
	 * @return string      The URL.
	 */
	public static function getTargetUrl( $url ) {
		$array = explode( '?', trim( $url ) );
		$url   = '/' === substr( $array[0], 0, 1 ) ? user_trailingslashit( $array[0] ) : $array[0];

		return ! empty( $array[1] ) ? $url . '?' . $array[1] : $url;
	}

	/**
	 * Format source URL.
	 *
	 * @since 1.0.0
	 *
	 * @param  string $url The original url.
	 * @return string      The formatted URL.
	 */
	public static function formatSourceUrl( $url ) {
		$complete = home_url() . '/' . trim( $url );
		list( $uprotocol, $uempty, $uhost, $from ) = explode( '/', $complete, 4 ); // phpcs:ignore VariableAnalysis.CodeAnalysis.VariableAnalysis.UnusedVariable
		$from = '/' . $from;
		return str_replace( '//', '/', rtrim( $from, '/' ) );
	}

	/**
	 * Format target URL.
	 *
	 * @since 1.0.0
	 *
	 * @param  string $url The original url.
	 * @return string      The formatted URL.
	 */
	public static function formatTargetUrl( $url ) {
		$url = self::getTargetUrl( $url );
		if ( strpos( $url, 'http' ) === 0 || strpos( $url, 'ftp' ) === 0 ) {
			return $url;
		}

		return rtrim( home_url(), '/' ) . '/' . ltrim( $url, '/' );
	}

	/**
	 * Get the user agent from the request.
	 *
	 * @since 1.0.0
	 *
	 * @return string The user agent.
	 */
	public static function getUserAgent() {
		$agent = '';

		if ( isset( $_SERVER['HTTP_USER_AGENT'] ) ) {
			$agent = $_SERVER['HTTP_USER_AGENT'];
		}

		return apply_filters( 'aioseo_redirects_request_agent', $agent );
	}

	/**
	 * Get the referrer from the request.
	 *
	 * @since 1.0.0
	 *
	 * @return string The referrer.
	 */
	public static function getReferrer() {
		$referrer = '';

		if ( isset( $_SERVER['HTTP_REFERER'] ) ) {
			$referrer = wp_unslash( $_SERVER['HTTP_REFERER'] ); // phpcs:ignore HM.Security.ValidatedSanitizedInput.InputNotSanitized
		}

		return apply_filters( 'aioseo_redirects_request_referrer', $referrer );
	}

	/**
	 * Get the request method from the request.
	 *
	 * @since 1.0.0
	 *
	 * @return string The request method.
	 */
	public static function getRequestMethod() {
		$method = '';

		if ( isset( $_SERVER['REQUEST_METHOD'] ) ) {
			$method = $_SERVER['REQUEST_METHOD'];
		}

		return apply_filters( 'aioseo_redirects_request_method', $method );
	}

	/**
	 * Get the headers from the request.
	 *
	 * @since 1.0.0
	 *
	 * @return string The headers.
	 */
	public static function getRequestHeaders() {
		$ignore = apply_filters( 'aioseo_redirects_request_headers_ignore', [
			'cookie',
			'host',
		] );
		$headers = [];

		foreach ( $_SERVER as $name => $value ) {
			if ( substr( $name, 0, 5 ) === 'HTTP_' ) {
				$name = strtolower( $name );
				if ( in_array( $name, array_map( 'strtolower', self::getIpHeaders() ), true ) ) {
					if ( ! aioseoRedirects()->options->logs->ipAddress->enabled ) {
						continue;
					}

					$ipLevel = json_decode( aioseoRedirects()->options->logs->ipAddress->level )->value;
					$value   = 'full' === $ipLevel ? $value : self::maskIp( $value );
				}

				$name = substr( $name, 5 );
				$name = str_replace( '_', ' ', $name );
				$name = ucwords( $name );
				$name = str_replace( ' ', '-', $name );

				if ( ! in_array( strtolower( $name ), $ignore, true ) ) {
					$headers[ $name ] = $value;
				}
			}
		}

		return apply_filters( 'aioseo_redirects_request_headers', $headers );
	}

	/**
	 * Get the IP headers.
	 *
	 * @since 1.0.0
	 *
	 * @return string The IP headers.
	 */
	public static function getIpHeaders() {
		return [
			'HTTP_CF_CONNECTING_IP',
			'HTTP_CLIENT_IP',
			'HTTP_X_FORWARDED_FOR',
			'HTTP_X_FORWARDED',
			'HTTP_X_CLUSTER_CLIENT_IP',
			'HTTP_X_REAL_IP',
			'HTTP_FORWARDED_FOR',
			'HTTP_FORWARDED',
			'HTTP_VIA',
			'REMOTE_ADDR',
		];
	}

	/**
	 * Get the IP address from the request.
	 *
	 * @since 1.0.0
	 *
	 * @return string The IP address.
	 */
	public static function getIp() {
		$ip = '';

		foreach ( self::getIpHeaders() as $var ) {
			if ( ! empty( $_SERVER[ $var ] ) ) {
				$ip = wp_unslash( $_SERVER[ $var ] ); // phpcs:ignore HM.Security.ValidatedSanitizedInput.InputNotSanitized
				$ip = explode( ',', $ip );
				$ip = array_shift( $ip );
				break;
			}
		}

		// Convert to binary.
		$ip = @inet_pton( trim( $ip ) );
		if ( false !== $ip ) {
			$ip = @inet_ntop( $ip ); // Convert back to string.
		}

		return $ip ? $ip : '';
	}

	/**
	 * Masks the IP address passed in.
	 *
	 * @since 1.0.0
	 *
	 * @param  string $ip The IP Address to mask.
	 * @return string     The masked IP address.
	 */
	public static function maskIp( $ip ) {
		$ip = trim( $ip );
		if ( false !== strpos( $ip, ':' ) ) {
			$ip = @inet_pton( trim( $ip ) );

			return @inet_ntop( $ip & pack( 'a16', 'ffff:ffff:ffff:ffff::ff00::0000::0000::0000' ) );
		}

		$parts = [];
		if ( strlen( $ip ) > 0 ) {
			$parts = explode( '.', $ip );
		}

		if ( count( $parts ) > 0 ) {
			$parts[ count( $parts ) - 1 ] = 0;
		}

		return implode( '.', $parts );
	}

	/**
	 * Get the server.
	 *
	 * @since 1.0.0
	 *
	 * @return string The server.
	 */
	public static function getServer() {
		return self::getProtocol() . '://' . self::getServerName();
	}

	/**
	 * Get the protocol.
	 *
	 * @since 1.0.0
	 *
	 * @return string The protocol.
	 */
	public static function getProtocol() {
		return is_ssl() ? 'https' : 'http';
	}

	/**
	 * Get the server name (from $_SERVER['SERVER_NAME]), or use the request name ($_SERVER['HTTP_HOST']) if not present.
	 *
	 * @since 1.0.0
	 *
	 * @return string The server name.
	 */
	public static function getServerName() {
		$host = self::getRequestServerName();

		if ( isset( $_SERVER['SERVER_NAME'] ) ) {
			$host = wp_unslash( $_SERVER['SERVER_NAME'] ); // phpcs:ignore HM.Security.ValidatedSanitizedInput.InputNotSanitized
		}

		return apply_filters( 'aioseo_redirects_request_server', $host );
	}

	/**
	 * Get the request server name (from $_SERVER['HTTP_HOST]).
	 *
	 * @since 1.0.0
	 *
	 * @return string The request server name.
	 */
	public static function getRequestServerName() {
		$host = '';

		if ( isset( $_SERVER['HTTP_HOST'] ) ) {
			$host = $_SERVER['HTTP_HOST'];
		}

		return apply_filters( 'aioseo_redirects_request_server_host', $host );
	}
}